#include <Adafruit_NeoPixel.h>

// Variables
#define LED_PIN    6
#define LED_COUNT 60

int index; // Stores the current index
int r, g, b; // Stores the current color values
int cubeIndex; // Stores the start led for a cubes leds

// Length of the longest message this code is able to receive
const int arrayLength = 32;

// stores the message received over serial
char message[arrayLength];

// When true, you have obtained a new message and are ready to convert
// into servo angles.
boolean haveNewData = false;

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

void setup() {

  // Starts Serial and sets the BAUD Rate to 9600
  Serial.begin(9600);

  strip.begin();           // INITIALIZE NeoPixel strip object
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(255); // Set BRIGHTNESS (max = 255)

}

void receive() {

  // becomes true if we are currently receiving a message
  static boolean rxInProgress = false;
  // current the index of array "message"
  static int index = 0;
  // marks the start of a message
  char startMarker = '<';
  // marks the end of a message
  char endMarker = '>';
  // stores the character we have just read from serial
  char currentChar;

  while (Serial.available() > 0 && haveNewData == false) {
    currentChar = Serial.read();
    if (rxInProgress) {
      if (endMarker != currentChar) {
        message[index] = currentChar;
        //Serial.println(currentChar);
        index++;
      }
      else {
        message[index] = '\0';
        rxInProgress = false;
        index = 0;
        haveNewData = true;
      }
    }

    else if (startMarker == currentChar) {
      rxInProgress = true;
    }
  }
}

void updateLeds() {

  if (haveNewData) {
    // converts the array of characters into a String
    String messageString = String(message);
    // Sets this equal to the index of the first comma in the string
    int comma1Index = messageString.indexOf(',');
    // Parses the string
    String indexString = messageString.substring(0, comma1Index);
    // Stores the parsed string in an int variable
    int newIndex = indexString.toInt();
    // receives the index to change
    index = newIndex;

    messageString = messageString.substring(comma1Index + 1);

    // Sets this equal to the index of the first comma in the string
    int comma2Index = messageString.indexOf(',');
    // Parses the string
    String rString = messageString.substring(0, comma2Index);
    // Stores the parsed string in an int variable
    int newR = rString.toInt();
    // stores the red value to change
    r = newR;

    messageString = messageString.substring(comma2Index + 1);

    // Sets this equal to the index of the first comma in the string
    int comma3Index = messageString.indexOf(',');
    // Parses the string
    String gString = messageString.substring(0, comma3Index);
    // Stores the parsed string in an int variable
    int newG = gString.toInt();
    // stores the green value to change
    g = newG;

    messageString = messageString.substring(comma3Index + 1);

    // Sets this equal to the index of the first comma in the string
    int comma4Index = messageString.indexOf(',');
    // Parses the string
    String bString = messageString.substring(0, comma4Index);
    // Stores the parsed string in an int variable
    int newB = bString.toInt();
    // stores the blue value to change
    b = newB;

    // sets a cubes leds to a certain color

    cubeIndex = index * 5;
    
    for (int led = cubeIndex; led < cubeIndex + 5; led++) {
      strip.setPixelColor(led, r, g, b);
    }
    strip.show();
    

  }

}

void printNewData() {

  if (haveNewData == true) {
    haveNewData = false;
    Serial.println(message);
    strip.show();
  }
}


void loop() {

  receive();
  updateLeds();
  printNewData();

}
